﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WavesLine_AWG8;

namespace AwgDebugTool
{
    public partial class MainForm : Form
    {
        Usb3Device device = new Usb3Device();
        #region [ Constructor/ Device Search ]
        public MainForm()
        {
            InitializeComponent();

            SearchDevice();
            if (combo_DeviceList.Items.Count < 1)
            {
                MessageBox.Show("No Device(s) Detected.", "AWG", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            #region < Idn >
            string idn;
            if (!device.QueryIdn(out idn))
                MessageBox.Show("Fail To Set Get Devie Info", "AWG",
                            MessageBoxButtons.OK, MessageBoxIcon.Warning);

            lab_DeviceInfo.Text = idn;
            #endregion

            QueryStatus();

            this.FormClosing += MainForm_FormClosing;
        }

        void QueryStatus()
        {
            UInt64 f = 0;
            if (device.QueryFrequecy(out f) == false)
            {
                MessageBox.Show("Uable To Get Device Frequency.", "AWG", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            numeric_Frequency.Value = f;

            Int32 p = -175;
            if (device.QueryPower(out p) == false)
            {
                MessageBox.Show("Uable To Get Device Power.", "AWG", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            numeric_Power.Value = p / 100;

            if (device.QueryState(out device.OUTPUT, out device.BBG, out device.EXTREF) == false)
            {
                MessageBox.Show("Uable To Get Device Info.", "AWG", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            check_RfOutput.Checked = device.OUTPUT;
            check_ExtRef.Checked = device.EXTREF;
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            //Close device and release remote control
            if (!device.Close())
                MessageBox.Show("Fail To Close Device.", "AWG", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        void SearchDevice()
        {
            device.GetConnectedDevices();

            Collection<string> serials;
            if (!device.GetDeviceSerial(out serials)) return;
            combo_DeviceList.Items.Clear();
            for (int i = 0; i < serials.Count; i++)
                combo_DeviceList.Items.Add("[" + i.ToString() + "] " + serials[i]);

            if (serials.Count == 1)
                combo_DeviceList.SelectedIndex = 0;
        }

        private void combo_DeviceList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (device.Handel != (IntPtr)0)
            {
                device.Close();
            }

            int index = combo_DeviceList.SelectedIndex;
            if (!device.GetHandelFromIndex(index)) return;
            if (!device.Open(true)) return;

            lab_DeviceLink.Text = "[" + index.ToString() + "]# of " +
                device.NumOfActiveDevices.ToString() + " Device(s) Linked.";
        }
        #endregion

        #region [ Output/Frequency/Power ]
        private void check_RfOutput_CheckedChanged(object sender, EventArgs e)
        {
            device.OUTPUT = check_RfOutput.Checked;
            if(!device.SetOutput())
            {
                MessageBox.Show("Fail To Set Output.", "AWG",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void numeric_Frequency_ValueChanged(object sender, EventArgs e)
        {
            if(check_MOD.Checked)
            {
                MessageBox.Show("Please Stop BBG Then Set Frequency.", "AWG",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            if (!device.SetFrequency((UInt64)(numeric_Frequency.Value)))
            {
                MessageBox.Show("Fail To Set Frequency.", "AWG",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void numeric_Power_ValueChanged(object sender, EventArgs e)
        {
            if (!device.SetPower((int)(numeric_Power.Value * 100)))
            {
                MessageBox.Show("Fail To Set Power.", "AWG",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
        #endregion

        #region [ QMC ]
        void SetGainPhase()
        {
            if (!device.QmcSetCor((Int16)numeric_QMC_GainI.Value,
                                  (Int16)numeric_QMC_GainQ.Value,
                                  (Int16)numeric_QMC_Phase.Value))
            {
                MessageBox.Show("Fail To Set QMC Cor.", "QMC",
                   MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        void SetOffset()
        {
            if (!device.QmcSetOfs((int)numeric_Qmc_OffsetI.Value,
                                  (int)numeric_Qmc_OffsetQ.Value))
            {
                MessageBox.Show("Fail To Set QMC OFS.", "QMC",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
        private void numeric_QMC_GainI_ValueChanged(object sender, EventArgs e)
        {
            SetGainPhase();
        }

        private void numeric_QMC_GainQ_ValueChanged(object sender, EventArgs e)
        {
            SetGainPhase();
        }

        private void numeric_QMC_Phase_ValueChanged(object sender, EventArgs e)
        {
            SetGainPhase();
        }

        private void numeric_Qmc_OffsetI_ValueChanged(object sender, EventArgs e)
        {
            SetOffset();
        }


        private void numeric_Qmc_OffsetQ_ValueChanged(object sender, EventArgs e)
        {
            SetOffset();
        }
        #endregion
        
        #region [ Modulation  Control ]        
        OpenFileDialog ofd = new OpenFileDialog();
        private void bLoadFile_Click(object sender, EventArgs e)
        {
            if (backgroundWorker1.IsBusy) return;

            ofd.RestoreDirectory = true;
            ofd.Filter = "WaveForm File (*.wfm)|*.wfm";
            if (ofd.ShowDialog() != DialogResult.OK) return;
            
            textBox_FilePath.Text = ofd.FileName;
            device.LoadWfmFile(ofd.FileName);
            
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            device.PlayWfm(backgroundWorker1);
            device.SetMod(true);
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressBar1.Value = e.ProgressPercentage;
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            numeric_QMC_GainI.Value = device.iGain;
            numeric_QMC_GainQ.Value = device.qGain;
            numeric_QMC_Phase.Value = device.Qmcphase;

            numeric_Qmc_OffsetI.Value = device.iOfs;
            numeric_Qmc_OffsetQ.Value = device.qOfs;
        }

        private void check_MOD_CheckedChanged(object sender, EventArgs e)
        {
            if (check_MOD.Checked)
            {//start BBG
                if (backgroundWorker1.IsBusy)
                {
                    MessageBox.Show("Thread Busy.");
                    return;
                }

                progressBar1.Value = 0;
                backgroundWorker1.RunWorkerAsync();
            }
            else
            {//stop BBG
                device.SetMod(false);
                device.BbgStop();
            }
        }
        #endregion

        #region [ Ext REF ]
        private void check_ExtRef_CheckedChanged(object sender, EventArgs e)
        {
            device.EXTREF = check_ExtRef.Checked;
            device.SetRef();
        }
        #endregion
    }
}
